'use strict';
process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
var fs = require('fs'),
    configFilepath = require('path'),
    shell = require('shelljs'),
    chalk = require('chalk'),
    request = require('request'),
    querystring = require('querystring'),
    cliVersion = require('../package').version,
    prompt = require('prompt'),
    _ = require('lodash'),
    exec = require('child_process').exec,
    os = require('os');


/* Breaking functionality into submodules */
var utils = require('./utils');
//install = require('./install');
var genrichFolder = __dirname + '/../../genrich-data/';
var configFile = genrichFolder + 'genrich.json';
var genrichSecret = genrichFolder + 'genrich-secrect';
var enrichWorkspace = '/var/www/html/enrich';


var extraRequiredCommands = 'mkdir -p ' + genrichFolder
                            + '&& git config --global core.fileMode false'
                            + '&& mkdir -p ' + enrichWorkspace
                            + '&& chmod -R 0777 ' + enrichWorkspace;

exec(extraRequiredCommands, function(error, stdout, stderr) {
    if (error !== null) {
        console.log(chalk.red('genrich should be run with root/sudo permision'));
        console.log(stderr);
        return;
    }
});



function promptSetting(callback) {
    prompt.get([{
        name: 'bitbucketUsername',
        description: 'Your bitbucket username?',
        type: 'string',
        required: true
    },{
        name: 'bitbucketPassword',
        description: 'Your bitbucket password?',
        type: 'string',
        required: true,
        hidden: true
    },{
        name: 'beanstalkUsername',
        description: 'Your beanstalk username?',
        type: 'string',
        required: true
    },{
        name: 'beanstalkPassword',
        description: 'Your beanstalk password?',
        type: 'string',
        required: true,
        hidden: true
    }], function (err, results) {
        var configData = {
            bitbucket: {
                username: results.bitbucketUsername,
                password: results.bitbucketPassword
            },
            beanstalk: {
                username: results.beanstalkUsername,
                password: results.beanstalkPassword
            }
        };
        fs.writeFile(configFile,JSON.stringify(configData),function(err){
            if(err)
                console.log('Got problem while writing configuration', err);
        });

        callback(configData);
    });    
}

function getConfiguration(callback) {
    if (!fs.existsSync(configFile)) {
        promptSetting(callback);
    } else {
        fs.readFile(configFile, 'utf8', function(err, data){
            if(err) {
                promptSetting(callback);
            } else {
                try {
                    var dataJson = JSON.parse(data);
                    callback(dataJson);
                }catch(e) {
                    promptSetting(callback);
                }
            }
        });
    }
}

function getEncodePassword(password) {
    password = encodeURIComponent(password);

    return password.replace(/@/g, '%40')
                    .replace(/:/g, '%3A')
                    .replace(/!/g, '%21')
                    .replace(/#/g, '%23')
                    .replace(/\$/g, '%24');
}

function getProjects() {
    var projects = JSON.parse(fs.readFileSync(genrichSecret + '/projects.json').toString());
    return projects;
}

function displayProjects() {
    var projects = getProjects();
    console.log(chalk.green('Here is all projects for you: '));
    _.each(projects, function(item) {
        console.log('\t - ' + item);
    });
}

function makeRequiredFileAndFolder(project, callback) {
    fs.readFile(genrichSecret + '/' + project +  '/required', 'utf8', function(err, data){
        var listOfRequired = data.split('\n');
        _.each(listOfRequired, function(item) {
            var requiredObject = item.split(':');
            if(requiredObject.length != 2)
                return;

            if(requiredObject[0] == 'folder') {
                if (!fs.existsSync(requiredObject[1]))
                    fs.mkdirSync(requiredObject[1]);
            } else {
                if (!fs.existsSync(requiredObject[1]))
                    fs.writeFileSync(requiredObject[1], '', 'utf8');
            }
        });

        callback();
    });
}

function getProjectBuiltInfo(project) {
    var info = fs.readFileSync(genrichSecret + '/' + project +  '/info').toString();
    
    return info;
}

exports.projects = function (name, options) {
    getConfiguration(function(data) {
        var encodePassword = getEncodePassword(data.bitbucket.password);
        var genrichSecretGit = 'https://'+ data.bitbucket.username + ':' + encodePassword + '@bitbucket.org/enrichdevelopers/genrich-secret.git';
        if (!fs.existsSync(genrichSecret)) {
            exec('git clone ' + genrichSecretGit + ' ' + genrichSecret, function(error, stdout, stderr) {
                if (error !== null) {
                    console.log(chalk.red('Your bitbucket credential is incorrect!'));
                    console.log(chalk.yellow('Run "genrich configure" to update the credential'));
                    return;
                }

                displayProjects();
            });
        } else {
            displayProjects();
        }
    });
};

exports.update = function (options) {
    getConfiguration(function(data) {
        var encodePassword = getEncodePassword(data.bitbucket.password);

        var genrichSecretGit = 'https://'+ data.bitbucket.username + ':' + encodePassword + '@bitbucket.org/enrichdevelopers/genrich-secret.git';

        var jobUpdateExec;
        
        if (!fs.existsSync(genrichSecret)) {
            jobUpdateExec = exec('git clone ' + genrichSecretGit + ' ' + genrichSecret, {
                                                    timeout: 0, 
                                                    maxBuffer: 999999 * 1024 
                                                });
        } else {
            jobUpdateExec = exec('cd ' + genrichSecret + ' && git add -A && git reset --hard && git pull', {
                                                    timeout: 0, 
                                                    maxBuffer: 999999 * 1024 
                                                });
        }

        jobUpdateExec.stdout.on('data', function(data) {
            console.log(chalk.blue(data));
        });

        jobUpdateExec.stderr.on('data', function(data) {
            console.log(chalk.red(data));
        });

        jobUpdateExec.on('exit', function(code) {
            if(code != 0) {
                console.log(chalk.red('Your bitbucket credential is incorrect!'));
                console.log(chalk.yellow('Run "genrich configure" to update the credential'));
                return;
            }

            console.log(chalk.green('Update data was done!'));
        });
    });
};

exports.configure = function(options) {
    if(!options.list && !options.update) {
        console.error(chalk.red('ERROR: Action is required'));
        console.error(chalk.blue('\t -l to display all credentials'));
        console.error(chalk.blue('\t -u to update credentials'));
        
        return;
    }

    if(options.list) {
        getConfiguration(function(data) {
            var output = '\t Bitbucket:\n'
                        + '\t\t-Username: ' + data.bitbucket.username + '\n'
                        + '\t\t-Password: ' + data.bitbucket.password + '\n'
                        + '\t Beanstalk:\n'
                        + '\t\t-Username: ' + data.beanstalk.username + '\n'
                        + '\t\t-Password: ' + data.beanstalk.password + '\n'
            console.log(chalk.green('Your credentials: \n'), output);
        });
    } else {
        promptSetting(function() {
            console.log(chalk.green('Your settings were done'));
        });
    }
}

exports.project = function(project, options) {
    if(!options.install && !options.view) {
        console.error(chalk.red('ERROR: Action is required'));
        console.error(chalk.blue('\t -i or --install to install'));
        console.error(chalk.blue('\t -v or view  to see the information of built project'));
        console.error(chalk.blue('\t -u to switch to project'));
        console.error(chalk.blue('\t -u to switch to project'));
        return;
    }

    var projects = getProjects();

    if(projects.indexOf(project) == -1) {
        console.error(chalk.red('ERROR: project named "' + project + '" is not existing'));
        console.error(chalk.blue('INFO: run "genrich projects" to see available projects'));
        return;
    }

    if(options.install) {
        console.log(chalk.green('We are still setting your environment for "' + project + '"...'));


        if(os.type() == 'Darwin') {
            exec('ifconfig en0 alias 10.254.254.254 255.255.255.0', function(error, stdout, stderr) {
                if (error !== null) {
                    console.log(chalk.red('genrich should be run with root/sudo permision'));
                    return;
                }
            });    
        } else if(os.type() == 'Linux'){
            exec('ifconfig | grep -Eo "enp[^ ]+"', function(error, stdout, stderr) {
                if (error !== null) {
                    console.log(chalk.red('genrich should be run with root/sudo permision'));
                    return;
                }

                var eth = stdout.replace(/\:/g, '')
                                .replace(/\s/g, '');

                if(stdout.indexOf('genrich') == -1) {
                    exec('ip addr add 10.254.254.254/24 dev ' + eth + ' label ' + eth + ':genrich', function(error, stdout, stderr) {
                        if (error !== null) {
                            console.log(chalk.red('genrich should be run with root/sudo permision'));
                            return;
                        }
                    });   
                }
            });
        }
        
        switch(project) {
            case "example": 
                makeRequiredFileAndFolder(project, function() {
                    var exampleProjectWorkspace = enrichWorkspace + '/example';

                    var jobExec = exec('cd ' + genrichSecret + '/' + project 
                                             + ' && cp code/index.php ' 
                                             + enrichWorkspace + '/example/index.php  && docker-compose up --build -d', {
                                                timeout: 0, 
                                                maxBuffer: 999999 * 1024 
                                             });
                    jobExec.stdout.on('data', function(data) {
                        console.log(chalk.blue(data));
                    });

                    jobExec.stderr.on('data', function(data) {
                        console.log(chalk.red(data));
                    });

                    jobExec.on('exit', function(code) {
                        console.log('exit with code', code);
                        if(code != 0) {
                            console.log(chalk.red('Got problem while setting up project named "' + project + '"'));
                            return;
                        }
                        var projectInfo = getProjectBuiltInfo(project);
                        
                        console.log(chalk.green('Installing example project was done'));
                        console.log(chalk.green(projectInfo));
                    });
                });

                break;
            case "petbarn": 
                makeRequiredFileAndFolder(project, function() {
                    var petbarnProjectWorkspace = enrichWorkspace + '/petbarn';
                    getConfiguration(function(data) {

                        if (!fs.existsSync(petbarnProjectWorkspace + '/.git')){
                            var encodePassword = getEncodePassword(data.beanstalk.password);
                            var petbarnGit = 'https://'+ data.beanstalk.username + ':' + encodePassword + '@epartnerdigital.git.beanstalkapp.com/petbarn.git -b develop';
                            
                            console.log(chalk.blue('Start cloning Petbarn source code...'));

                            exec('cd ' + petbarnProjectWorkspace 
                                    + ' && rm -rf *'
                                    + ' && git clone ' + petbarnGit + ' .'
                                    + ' && chmod -R 0777 ' + petbarnProjectWorkspace,{
                                                    timeout: 0, 
                                                    maxBuffer: 999999 * 1024 
                                                }, function(error, stdout, stderr) {
                                if (error !== null) {
                                    if(options.debug) {
                                        console.log(chalk.red(stderr));    
                                    }
                                    console.log(chalk.red('Your Beanstalk credential is incorrect!'));
                                    console.log(chalk.yellow('Run "genrich configure" to update the credential'));
                                    return;
                                }

                                nextStep();
                            });
                        } else {
                            nextStep();
                        }

                        
                        function nextStep() {

                            console.log(chalk.blue('Start setting docker for Petbarn project...'));

                            var jobExec = exec('cd ' + genrichSecret + '/' + project 
                                                + '  &&  docker-compose pull'
                                                + '  &&  docker-compose build --no-cache'
                                                + '  &&  docker-compose up -d --force-recreate'
                                                + '  &&  echo "Importing database, please wait..."'
                                                + '  &&  sleep 10'
                                                + ' && docker exec petbarn_db_1 sh /tmp/setup.sh', {
                                                    timeout: 0, 
                                                    maxBuffer: 999999 * 1024 
                                                });
                            jobExec.stdout.on('data', function(data) {
                                console.log(chalk.blue(data));
                            });

                            jobExec.stderr.on('data', function(data) {
                                console.log(chalk.red(data));
                            });

                            jobExec.on('exit', function(code) {
                                console.log('exit with code', code);
                                if(code != 0) {
                                    console.log(chalk.red('Got problem while setting up project named "' + project + '"'));
                                    return;
                                }
                                var projectInfo = getProjectBuiltInfo(project);
                                
                                console.log(chalk.green('Installing example project was done'));
                                console.log(chalk.green(projectInfo));
                            });
                        }
                    });
                });
                break;

            case "esprit": 
                makeRequiredFileAndFolder(project, function() {
                    var espritProjectWorkspace = enrichWorkspace + '/esprit';
                    getConfiguration(function(data) {

                        if (!fs.existsSync(espritProjectWorkspace + '/.svn')){
                            var espritSVN = '--non-interactive --no-auth-cache --username '+ data.beanstalk.username + ' --password ' + data.beanstalk.password + ' https://epartnerdigital.svn.beanstalkapp.com/esprit/trunk';
                            
                            console.log(chalk.blue('Start cloning Esprit source code...'));

                            exec('cd ' + espritProjectWorkspace 
                                        + ' && rm -rf *'
                                        + ' && svn co ' + espritSVN + ' .',{
                                                    timeout: 0, 
                                                    maxBuffer: 999999 * 1024 
                                                }, function(error, stdout, stderr) {
                                if (error !== null) {
                                    if(options.debug) {
                                        console.log(chalk.red(stderr));    
                                    }
                                    console.log(chalk.red('Your Beanstalk credential is incorrect!'));
                                    console.log(chalk.yellow('Run "genrich configure" to update the credential'));
                                    return;
                                }
                                espritNextStep();
                            });
                        } else {
                            espritNextStep();
                        }

                        function espritNextStep() {
                            console.log(chalk.blue('Start setting docker for Esprit project...'));

                            var jobExec = exec('chmod -R 0777 ' + espritProjectWorkspace
                                                    + ' && cd ' + genrichSecret + '/' + project 
                                                    + ' && docker-compose pull'
                                                    + ' && docker-compose build --no-cache'
                                                    + ' && docker-compose up -d --force-recreate'
                                                    + ' && docker exec esprit_db_1 /usr/bin/mysqldump -v -u root --password=root esprit-dev < /tmp > /tmp/restore.log', {
                                                        timeout: 0, 
                                                        maxBuffer: 999999 * 1024 
                                                    });
                            jobExec.stdout.on('data', function(data) {
                                console.log(chalk.blue(data));
                            });

                            jobExec.stderr.on('data', function(data) {
                                console.log(chalk.red(data));
                            });

                            jobExec.on('exit', function(code) {
                                console.log('exit with code', code);
                                if(code != 0) {
                                    console.log(chalk.red('Got problem while setting up project named "' + project + '"'));
                                    return;
                                }
                                var projectInfo = getProjectBuiltInfo(project);
                                
                                console.log(chalk.green('Installing example project was done'));
                                console.log(chalk.green(projectInfo));
                            });
                        }
                    });
                });
                break;

            case "jeanswest": 
                makeRequiredFileAndFolder(project, function() {
                    var jeanswestProjectWorkspace = enrichWorkspace + '/jeanswest';
                    getConfiguration(function(data) {

                        if (!fs.existsSync(jeanswestProjectWorkspace + '/.git')){
                            var encodePassword = getEncodePassword(data.beanstalk.password);

                            var jeanswestGit = 'https://'+ data.beanstalk.username + ':' + encodePassword + '@epartnerdigital.git.beanstalkapp.com/jeanswest.git -b develop';
                            
                            console.log(chalk.blue('Start cloning Jeanswest source code...'));

                            exec('cd ' + jeanswestProjectWorkspace 
                                    + ' && rm -rf *'
                                    + ' && git clone ' + jeanswestGit + ' .',{
                                                    timeout: 0, 
                                                    maxBuffer: 999999 * 1024 
                                                }, function(error, stdout, stderr) {
                                if (error !== null) {
                                    if(options.debug) {
                                        console.log(chalk.red(stderr));    
                                    }
                                    console.log(chalk.red('Your Beanstalk credential is incorrect!'));
                                    console.log(chalk.yellow('Run "genrich configure" to update the credential'));
                                    return;
                                }

                                jeanswestNext();
                            });
                        } else {
                            jeanswestNext();
                        }

                        function jeanswestNext() {
                            console.log(chalk.blue('Start setting docker for Jeanswest project...'));

                            var jobExec = exec( 'chmod -R 0777 ' + jeanswestProjectWorkspace
                                                    + ' && cd ' + jeanswestProjectWorkspace
                                                    + ' && git config core.fileMode false'
                                                    + ' && cd ' + genrichSecret + '/' + project 
                                                    + ' && docker-compose pull'
                                                    + ' && docker-compose build --no-cache'
                                                    + ' && docker-compose up -d --force-recreate', {
                                                        timeout: 0, 
                                                        maxBuffer: 999999 * 1024 
                                                    });
                            jobExec.stdout.on('data', function(data) {
                                console.log(chalk.blue(data));
                            });

                            jobExec.stderr.on('data', function(data) {
                                console.log(chalk.red(data));
                            });

                            jobExec.on('exit', function(code) {
                                console.log('exit with code', code);
                                if(code != 0) {
                                    console.log(chalk.red('Got problem while setting up project named "' + project + '"'));
                                    return;
                                }
                                var projectInfo = getProjectBuiltInfo(project);
                                
                                console.log(chalk.green('Installing example project was done'));
                                console.log(chalk.green(projectInfo));
                            });
                        }
                    });
                });
                break;
            
            default:
                console.log(chalk.green('Do nothing'));
                break;
        }
    } else if (options.view) {

        exec('cd ' + genrichSecret + '/' + project + ' && docker-compose ps -q', function(error, stdout, stderr) {
            if (error !== null) {
                console.log(error);
                return;
            }
            var dataResponse = stdout.split('\n');
            if(dataResponse.length < 2) {
                console.log(chalk.red('ERROR: Project named "' + project + '" was not installed'));
                console.log(chalk.blue('INFO: Please run "genrich project <name> -i" to install'));
                return;
            }

            var projectInfo = getProjectBuiltInfo(project);
            console.log(chalk.green(projectInfo));

        });
    }
}


exports.docker = function(name, options) {
    if(!options.ip) {
        console.error(chalk.red('ERROR: Action is required'));
        console.error(chalk.blue('\t -ip or --ip to display ip of container'));
        
        return;
    }

    if(options.ip) {
        if(!name) {
            console.error(chalk.red('ERROR: Container ID is required'));
            return;
        }

        exec('docker inspect --format "{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}}" ' + name, function(error, stdout, stderr) {
            if (error !== null) {
                console.log(chalk.red('genrich should be run with root/sudo permision'));
                if(options.debug) {
                    console.log(chalk.red(stderr));
                }

                return;
            }

            console.log(chalk.blue(stdout));
        });

    }
}